@@ -1,5 +1,5 @@
-from typing import Union, Tuple, Dict
 from datetime import date, datetime, time
+from typing import Union, Tuple, Dict, Optional
 
-SQLParameter = Union[str, int, float, date, datetime, time]
+SQLParameter = Optional[Union[str, int, float, date, datetime, time, bool, bytes]]
 SQLParameters = Union[Tuple[SQLParameter, ...], Dict[str, SQLParameter], SQLParameter]
